from django.contrib.sites.models import Site
from django.utils.http import urlencode

from twitch_auth import app_settings


def get_absolute_uri(url):
    return app_settings.PROTOCOL + "%s%s" % (Site.objects.get_current().domain, url)


def build_url(url, extra_params):
    params = {
        'client_id': app_settings.CLIENT_ID,
        'scope': app_settings.SCOPE,
        'response_type': 'code'
    }
    params.update(extra_params)
    return "%s?%s" % (url, urlencode(params))



import logging
from django.contrib import messages
from django.contrib.auth import authenticate
from django.contrib.auth import login as auth_login
from django.contrib.auth import logout as auth_logout
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.utils.crypto import get_random_string
from django.utils.translation import ugettext_lazy as _
from django.http import HttpResponseRedirect
from django.views.decorators.csrf import csrf_protect

from twitch_auth import app_settings
from twitch_auth.utils import build_url

logger = logging.getLogger(__name__)


def login(request):
    redirect_to = request.GET.get('next', app_settings.REDIRECT_URI)
    if request.user.is_authenticated():
        return HttpResponseRedirect(redirect_to)
    authorize_url = 'https://api.twitch.tv/kraken/oauth2/authorize'
    params = {
        'redirect_uri': request.build_absolute_uri(reverse('callback_twitch')),
    }
    verifier = get_random_string()
    request.session['state'] = (redirect_to, verifier)
    params['state'] = verifier
    return HttpResponseRedirect(build_url(authorize_url, params))


def get_next_uri(request, verifier_ext):
    state, verifier = request.session.pop('state')
    if verifier != verifier_ext:
        raise PermissionDenied()
    return state


def callback(request):
    if 'error' in request.GET:
        error_message = _('You denied access to your Twitch account') \
            if request.GET['error'] == 'access_denied' else _('Error while authentication via Twitch API')
        messages.error(request, error_message)
        logger.error('[Twitch Auth] %s: %s' % (request.META['REMOTE_ADDR'], request.GET['error_description']))
        return HttpResponseRedirect('/')
    redirect_to = get_next_uri(request, request.GET['state'])
    if request.user.is_authenticated():
        return HttpResponseRedirect(redirect_to)
    user = authenticate(code=request.GET['code'])
    if user and user.is_active:
        auth_login(request, user)
        messages.success(request, _('You successfully logged in'))
        return HttpResponseRedirect(redirect_to)
    else:
        messages.error(request, _('Cannot authenticate you'))
        return HttpResponseRedirect('/')


@csrf_protect
def logout(request):
    auth_logout(request)
    messages.success(request, _('You successfully logged out'))
    return HttpResponseRedirect('/')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-03-06 09:54
from __future__ import unicode_literals

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import twitch_auth.fields


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='OAuth2AccessToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.TextField(help_text='access token (OAuth2)', verbose_name='token')),
                ('token_secret',
                 models.TextField(blank=True, help_text='refresh token (OAuth2)', verbose_name='token secret')),
                ('expires_at', models.DateTimeField(blank=True, null=True, verbose_name='expires at')),
            ],
            options={
                'verbose_name': 'oauth2 application token',
                'verbose_name_plural': 'oauth2 application tokens',
            },
        ),
        migrations.CreateModel(
            name='TwitchAccount',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uid', models.CharField(max_length=139, unique=True, verbose_name='uid')),
                ('extra_data', twitch_auth.fields.JSONField(default=dict, verbose_name='extra data')),
                (
                'user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'twitch account',
                'verbose_name_plural': 'twitch accounts',
            },
        ),
        migrations.AddField(
            model_name='oauth2accesstoken',
            name='account',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='twitch_auth.TwitchAccount'),
        ),
    ]



from __future__ import unicode_literals

from django.conf import settings
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from .fields import JSONField


@python_2_unicode_compatible
class TwitchAccount(models.Model):
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name='twitch'
    )

    uid = models.CharField(verbose_name=_('uid'), max_length=139, unique=True)
    extra_data = JSONField(verbose_name=_('extra data'), default=dict)

    class Meta:
        verbose_name = _('twitch account')
        verbose_name_plural = _('twitch accounts')

    def __str__(self):
        return self.user.username

    def get_profile_url(self):
        return 'https://twitch.tv/' + self.account.extra_data.get('name')

    def get_avatar_url(self):
        return self.account.extra_data.get('logo')


@python_2_unicode_compatible
class OAuth2AccessToken(models.Model):
    account = models.OneToOneField(TwitchAccount)
    token = models.TextField(
        verbose_name=_('token'),
        help_text=_(
            'access token (OAuth2)'))
    token_secret = models.TextField(
        blank=True,
        verbose_name=_('token secret'),
        help_text=_(
            'refresh token (OAuth2)'))
    expires_at = models.DateTimeField(blank=True, null=True,
                                      verbose_name=_('expires at'))

    class Meta:
        verbose_name = _('oauth2 application token')
        verbose_name_plural = _('oauth2 application tokens')

    def __str__(self):
        return self.token



from django.contrib import admin

from .models import TwitchAccount, OAuth2AccessToken


class TwitchAccountAdmin(admin.ModelAdmin):
    search_fields = ['user']
    raw_id_fields = ('user',)
    list_display = ('user', 'uid')


class OAuth2AccessTokenAdmin(admin.ModelAdmin):
    raw_id_fields = ('account',)
    list_display = ('account', 'truncated_token', 'expires_at')
    list_filter = ('expires_at',)

    def truncated_token(self, token):
        max_chars = 40
        ret = token.token
        if len(ret) > max_chars:
            ret = ret[0:max_chars] + '...(truncated)'
        return ret

    truncated_token.short_description = 'Token'


admin.site.register(TwitchAccount, TwitchAccountAdmin)
admin.site.register(OAuth2AccessToken, OAuth2AccessTokenAdmin)



from django.conf.urls import url

from . import views

urlpatterns = [
    url(r'^login/$', views.login, name='login_twitch'),
    url(r'^callback/$', views.callback, name='callback_twitch'),
    url(r'^logout/$', views.logout, name='logout'),
]



#!/usr/bin/env python
from setuptools import find_packages, setup

import twitch_auth

setup(
    name='django-simpletwitchauth',
    version=twitch_auth.__version__,
    description='Use this simple module to easily enable authorization in your Django application via Twitch API',
    long_description=open('README.rst').read(),
    url='https://github.com/ALikhachev/django-simpletwitchauth',
    author='Alexander Likhachev',
    author_email='likhachev96@gmail.com',
    install_requires=[
        'Django>=1.8',
        'requests'
    ],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    license='MIT License',
    keywords=['django auth authentication twitch external api'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)


import logging

import requests
from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse

from twitch_auth import app_settings
from twitch_auth.models import OAuth2AccessToken, TwitchAccount
from twitch_auth.utils import get_absolute_uri

logger = logging.getLogger(__name__)


class OAuth2Backend(ModelBackend):
    token_url = 'https://api.twitch.tv/kraken/oauth2/token'
    profile_url = 'https://api.twitch.tv/kraken/user'

    # We delegate the responsibility of authentication to the Twitch auth service
    # and only get the access_token from Twitch
    def authenticate(self, code=None):
        if code is None:
            logger.error('Auth code is not provided')
            return None
        resp = requests.post(
            self.token_url,
            data={
                'client_id': app_settings.CLIENT_ID,
                'client_secret': app_settings.CLIENT_SECRET,
                'grant_type': 'authorization_code',
                'code': code,
                'redirect_uri': get_absolute_uri(reverse('callback_twitch'))
            })
        try:
            data = resp.json()
        except ValueError:
            logger.error('Twitch auth service returned bad response: %s' % resp.content)
            return None

        if 'access_token' in data:
            token = data['access_token']
        else:
            logger.error('Twitch auth service returned bad response: %s' % data)
            return None

        user, twitch_account = self.get_user_profile(token)

        try:
            token_obj = OAuth2AccessToken.objects.get(account=twitch_account)
        except OAuth2AccessToken.DoesNotExist:
            token_obj = OAuth2AccessToken()
            token_obj.account = twitch_account
        token_obj.token = token
        token_obj.token_secret = data.get('refresh_token', '')
        token_obj.save()

        return user

    def get_user(self, user_id):
        try:
            return get_user_model().objects.get(pk=user_id)
        except User.DoesNotExist:
            return None

    def get_user_profile(self, token):
        resp = requests.get(self.profile_url,
                            params={'oauth_token': token, 'client_id': app_settings.CLIENT_ID})
        try:
            profile = resp.json()
        except ValueError:
            logger.error('Twitch profileservice returned bad response: %s' % resp.content)
            return None

        try:
            twitch_account = TwitchAccount.objects.get(uid=profile['_id'])
            user = twitch_account.user
        except TwitchAccount.DoesNotExist:
            twitch_account = TwitchAccount()
            user = get_user_model().objects.create(username=profile['display_name'])
            twitch_account.user = user
            twitch_account.uid = profile['_id']
        user.username = profile['display_name']
        user.email = profile.get('email', user.email)
        twitch_account.extra_data = profile
        twitch_account.save()
        user.save()
        return user, twitch_account



class AppSettings(object):
    def __init__(self, prefix):
        self.prefix = prefix

    def _setting(self, name, dflt):
        from django.conf import settings
        getter = getattr(settings,
                         'TWITCH_AUTH_SETTING_GETTER',
                         lambda name, dflt: getattr(settings, name, dflt))
        return getter(self.prefix + name, dflt)

    @property
    def SCOPE(self):
        return self._setting("SCOPE", "user_read")

    @property
    def PROTOCOL(self):
        return self._setting("PROTOCOL", "http://")

    @property
    def CLIENT_ID(self):
        return self._setting("CLIENT_ID", "")

    @property
    def CLIENT_SECRET(self):
        return self._setting("CLIENT_SECRET", "")

    @property
    def REDIRECT_URI(self):
        return self._setting("REDIRECT_URI", "/")


# Ugly? Guido recommends this himself ...
# http://mail.python.org/pipermail/python-ideas/2012-May/014969.html
import sys

app_settings = AppSettings('TWITCH_AUTH_')
app_settings.__name__ = __name__
sys.modules[__name__] = app_settings
